package com.jpattern.orm.datasource;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import com.jpattern.logger.ILogger;
import com.jpattern.orm.exception.OrmException;
import com.jpattern.orm.logger.OrmLogger;
import com.jpattern.orm.session.ASqlPerformer;
import com.jpattern.orm.session.IGeneratedKeyReader;
import com.jpattern.orm.session.IPreparedStatementCreator;
import com.jpattern.orm.session.IResultSetReader;

/**
 * 
 * @author Francesco Cina
 *
 * 02/lug/2011
 * 
 * ISqlExecutor implementation using java.sql.Connection as backend.
 */
public class DataSourceSqlPerformer extends ASqlPerformer implements IConnectionCaller {

	private final ILogger logger = OrmLogger.getOrmLogger(getClass());
	private int queryTimeout = 0;
	private int maxRows = 0;
	private final DataSourceSessionProvider dataSourceSessionProvider;

	public DataSourceSqlPerformer(DataSourceSessionProvider dataSourceSessionProvider) {
		this.dataSourceSessionProvider = dataSourceSessionProvider;
	}

	@Override
	public void setMaxRows(int maxRows) {
		logger.debug("setMaxRows", "max rows sets to " + maxRows);
		this.maxRows = maxRows;
	}

	@Override
	public int getMaxRows() {
		return maxRows;
	}

	@Override
	public void setQueryTimeout(int queryTimeout) {
		logger.debug("setQueryTimeout", "queryTimeout sets to " + queryTimeout);
		this.queryTimeout = queryTimeout;
	}

	@Override
	public int getQueryTimeout() {
		return queryTimeout;
	}

	@Override
	public void execute(String sql) throws OrmException {
		logger.debug("execute", "Method called");
		IPreparedStatement preparedStatement = null;
		IConnection conn = dataSourceSessionProvider.getConnection(false, this);
		try {
			preparedStatement = conn.prepareStatement( sql );
			preparedStatement.setQueryTimeout(getQueryTimeout());
			preparedStatement.execute();
			conn.commit();
		} catch (Exception e) {
			conn.rollback();
			throw new OrmException(e);
		} finally {
			try {
				if (preparedStatement!=null) {
					preparedStatement.close();
				}
			} catch (SQLException e) {
				throw new OrmException(e);
			} finally {
				conn.close(this);
			}
		}
	}

	@Override
	public <T> T query(String sql, IResultSetReader<T> rse, Object... args) throws OrmException {
			logger.debug("query", "Method called");
			ResultSet resultSet = null;
			IPreparedStatement preparedStatement = null;
			IConnection conn = dataSourceSessionProvider.getConnection(true, this);
			try {
				preparedStatement = conn.prepareStatement( sql );
				int i = 0;
				for (Object value : args) {
					preparedStatement.setObject(++i, value);
				}
				preparedStatement.setMaxRows(getMaxRows());
				preparedStatement.setQueryTimeout(getQueryTimeout());
				resultSet = preparedStatement.executeQuery();
				return rse.read(resultSet);
			} catch (Exception e) {
				throw new OrmException(e);
			} finally {
				try {
					if (resultSet!=null && !resultSet.isClosed()) {
						resultSet.close();
					}
					if (preparedStatement!=null) {
						preparedStatement.close();
					}
				} catch (SQLException e) {
					throw new OrmException(e);
				} finally {
					conn.close(this);
				}				
			}
	}

	@Override
	public int update(String sql, Object... args) throws OrmException {
		logger.debug("update", "Method called");
		IConnection conn = dataSourceSessionProvider.getConnection(false, this);
		IPreparedStatement preparedStatement = null;
		try {
			preparedStatement = conn.prepareStatement( sql );
			preparedStatement.setQueryTimeout(getQueryTimeout());
			int i = 0;
			for (Object value : args ) {
				preparedStatement.setObject(++i, value);
			}
			int result = preparedStatement.executeUpdate();
			conn.commit();
			return result;
		} catch (Exception e) {
			conn.rollback();
			throw new OrmException(e);
		} finally {
			try {
				if (preparedStatement!=null) {
					preparedStatement.close();
				}
			} catch (SQLException e) {
				throw new OrmException(e);
			} finally {
				conn.close(this);
			}
		}
	}

	@Override
	public int update(String sql, IGeneratedKeyReader generatedKeyExtractor, Object... args) throws OrmException {
		logger.debug("update", "Method called");
		IConnection conn = dataSourceSessionProvider.getConnection(false, this);
		ResultSet generatedKeyResultSet = null;
		IPreparedStatement preparedStatement = null;
		int result = 0;
		try {
			preparedStatement = conn.prepareStatement( sql , generatedKeyExtractor.generatedColumnNames());
			preparedStatement.setQueryTimeout(getQueryTimeout());
			int i = 0;
			for (Object value : args) {
				preparedStatement.setObject(++i, value);
			}
			result = preparedStatement.executeUpdate();
			generatedKeyResultSet = preparedStatement.getGeneratedKeys();
			generatedKeyExtractor.read(generatedKeyResultSet);
			conn.commit();
			return result;
		} catch (Exception e) {
			conn.rollback();
			throw new OrmException(e);
		} finally {
			try {
				if (preparedStatement!=null) {
					preparedStatement.close();
				}
				if (generatedKeyResultSet!=null && !generatedKeyResultSet.isClosed()) {
					generatedKeyResultSet.close();
				}
			} catch (SQLException e) {
				throw new OrmException(e);
			} finally {
				conn.close(this);
			}
		}
	}

	@Override
	public int[] batchUpdate(List<String> sqls) throws OrmException {
		logger.debug("batchUpdate", "Method called");
		IConnection conn = dataSourceSessionProvider.getConnection(false, this);
		IStatement statement = null;
		try {
			statement = conn.createStatement();
			statement.setQueryTimeout(getQueryTimeout());
			for (String sql : sqls) {
				statement.addBatch(sql);
			}
			int[] result = statement.executeBatch();
			conn.commit();
			return result;
		} catch (Exception e) {
			conn.rollback();
			throw new OrmException(e);
		} finally {
			try {
				if (statement!=null) {
					statement.close();
				}
			} catch (SQLException e) {
				throw new OrmException(e);
			} finally {
				conn.close(this);
			}
		}
	}

	@Override
	public int[] batchUpdate(String sql, List<Object[]> args) throws OrmException {
		logger.debug("batchUpdate", "Method called");
		IConnection conn = dataSourceSessionProvider.getConnection(false, this);
		IPreparedStatement preparedStatement = null;
		try {
			preparedStatement = conn.prepareStatement( sql );
			preparedStatement.setQueryTimeout(getQueryTimeout());
			for (Object[] arg : args) {
				int i = 0;
				for (Object value : arg) {
					preparedStatement.setObject(++i, value);
				}
				preparedStatement.addBatch();
			}
			int[] result = preparedStatement.executeBatch();
			conn.commit();
			return result;
		} catch (Exception e) {
			conn.rollback();
			throw new OrmException(e);
		} finally {
			try {
				if (preparedStatement!=null) {
					preparedStatement.close();
				}
			} catch (SQLException e) {
				throw new OrmException(e);
			} finally {
				conn.close(this);
			}
		}
	}

	@Override
	public int[] batchUpdate(String sql, IPreparedStatementCreator psc) throws OrmException {
		logger.debug("batchUpdate", "Method called");
		IConnection conn = dataSourceSessionProvider.getConnection(false, this);
		PreparedStatementWrapper preparedStatement = null;
		try {
			preparedStatement = conn.prepareStatement( sql );
			preparedStatement.setQueryTimeout(getQueryTimeout());
			for (int i=0; i<psc.getBatchSize(); i++) {
				psc.set(preparedStatement, i);
				preparedStatement.addBatch();
			}
			int[] result = preparedStatement.executeBatch();
			conn.commit();
			return result;
		} catch (Exception e) {
			conn.rollback();
			throw new OrmException(e);
		} finally {
			try {
				if (preparedStatement!=null) {
					preparedStatement.close();
				}
			} catch (SQLException e) {
				throw new OrmException(e);
			} finally {
				conn.close(this);
			}
		}
	}

}
